package com.fr.plugin.loginAuth.dbAccessProvider;

import com.fr.db.fun.impl.AbstractDBAccessProvider;
import com.fr.plugin.loginAuth.utils.FRUtils;
import com.fr.plugin.transform.FunctionRecorder;
import com.fr.record.analyzer.EnableMetrics;
import com.fr.stable.db.accessor.DBAccessor;
import com.fr.stable.db.action.DBAction;
import com.fr.stable.db.dao.DAOContext;
import com.fr.stable.db.dao.DAOProvider;

import java.util.ArrayList;
import java.util.List;

/**
 * @author 秃破天际
 * @version 10.0
 * Created by 秃破天际 on 2021-04-25
 **/
@EnableMetrics
@FunctionRecorder
public class AuthUserDBController extends AbstractDBAccessProvider {

    private static DBAccessor accessor;

    public static DBAccessor getAccessor() {
        return accessor;
    }

    @Override
    public DAOProvider[] registerDAO() {
        return new DAOProvider[]{
                AuthUserDao.DAO,
                AuthUserMacDAO.DAO
        };
    }

    @Override
    public void onDBAvailable(DBAccessor accessor) {
        AuthUserDBController.accessor = accessor;
    }


    public static List<AuthUserEntity> getByUserName(final String username ){
        try{
            return accessor.runQueryAction(new DBAction<List<AuthUserEntity>>() {
                @Override
                public List<AuthUserEntity> run(DAOContext context) throws Exception {
                    List<AuthUserEntity> list = context.getDAO(AuthUserDao.class).getByUserName(username);

                    return list;
                }
            });
        }catch(Throwable e){
            FRUtils.FRLogInfo("getByUserName:"+e.getMessage());
        }

        return new ArrayList<AuthUserEntity>();
    }

    public static List<AuthUserMacEntity> getUserAuthMac(final String username, final String mac) throws Exception{
        return accessor.runQueryAction(new DBAction<List<AuthUserMacEntity>>() {
            @Override
            public List<AuthUserMacEntity> run(DAOContext context) throws Exception {
                List<AuthUserMacEntity> list = context.getDAO(AuthUserMacDAO.class).getAuthUserMac(username,mac);

                return list;
            }
        });
    }

    public static List<AuthUserMacEntity> getUserAuthMac2(final String username, final String mac,final String deviceType) throws Exception{
        return accessor.runQueryAction(new DBAction<List<AuthUserMacEntity>>() {
            @Override
            public List<AuthUserMacEntity> run(DAOContext context) throws Exception {
                List<AuthUserMacEntity> list = context.getDAO(AuthUserMacDAO.class).getAuthUserMac2(username,mac,deviceType);

                return list;
            }
        });
    }

    /**
     * 业务对象——增
     * @param bean
     */
    public static void addUserAuth( final AuthUserEntity bean ) throws Exception {
        accessor.runDMLAction(new DBAction<Boolean>() {
            @Override
            public Boolean run(DAOContext context) throws Exception {
                context.getDAO(AuthUserDao.class).add(bean);
                return true;
            }
        });

    }

    /**
     * 业务对象——增
     * @param bean
     */
    public static void add( final AuthUserMacEntity bean ) throws Exception {
            accessor.runDMLAction(new DBAction<Boolean>() {
                @Override
                public Boolean run(DAOContext context) throws Exception {
                    context.getDAO(AuthUserMacDAO.class).add(bean);
                    return true;
                }
            });

    }
//    /**
//     * 获取org对象
//     * @param orgCode
//     * @return
//     */
//    public static AuthUserEntity getOrg(final String orgCode){
//        try{
//            return accessor.runQueryAction(new DBAction<AuthUserEntity>() {
//                @Override
//                public AuthUserEntity run(DAOContext context) throws Exception {
//                    List<AuthUserEntity> result = new ArrayList<AuthUserEntity>();
//                    List<AuthUserEntity> list = context.getDAO(AuthUserDao.class).likeByKey(orgCode);
//                    for( AuthUserEntity item : list ){
////                        result.add( toDataBean(item,null) );
//                    }
//                    return result != null && result.size() > 0 ? result.get(0) : new AuthUserEntity();
//                }
//            });
//        }catch(Throwable e){
////            LogKit.error(e,"Get DataBeans{} Error:{}",key,e.getMessage());
//        }
//        return new AuthUserEntity();
//    }
//
//    /**
//     * 业务对象——增
//     * @param bean
//     */
//    public static void add( final AuthUserEntity bean ){
//        try{
//            accessor.runDMLAction(new DBAction<Boolean>() {
//                @Override
//                public Boolean run(DAOContext context) throws Exception {
//                    context.getDAO(AuthUserDao.class).add( loadDemoEntity(bean,true) );
////                    context.getDAO(LinkDao.class).add( loadLinkEntities(bean,true) );
//                    return true;
//                }
//            });
//        }catch(Throwable e){
//            e.printStackTrace();
////            LogKit.error(e,"Add DataBean Error:{}",e.getMessage());
//        }
//    }
//
//    /**
//     * 业务对象——删
//     * @param id
//     */
//    public static AuthUserEntity deleteOrg( final String id ){
//        try{
//            return accessor.runDMLAction(new DBAction<AuthUserEntity>() {
//                @Override
//                public AuthUserEntity run(DAOContext context) throws Exception {
//                    AuthUserEntity entity = context.getDAO(AuthUserDao.class).getById(id);
//                    if( null == entity ){
//                        return null;
//                    }
//                    context.getDAO(AuthUserDao.class).remove( id );
////                    List<LinkEntity> list = context.getDAO(LinkDao.class).removeByKey(entity.getDid());
//                    return toDataBean(entity,null);
//                }
//            });
//        }catch(Throwable e){
////            LogKit.error(e,"Delete DataBean[{}] Error:{}",id,e.getMessage());
//        }
//        return null;
//    }
//
//    /**
//     * 业务对象——改
//     * @param bean
//     */
//    public static void update( AuthUserEntity bean ){
//        final AuthUserEntity bean2 = bean;
//        try{
//            accessor.runDMLAction(new DBAction<Boolean>() {
//                @Override
//                public Boolean run(DAOContext context) throws Exception {
//                    context.getDAO(AuthUserDao.class).update( loadDemoEntity(bean2,false) );
//                    context.getDAO(AuthUserMacDAO.class).addOrUpdate( loadLinkEntities(bean2,false) );
//                    return true;
//                }
//            });
//        }catch(Throwable e){
////            LogKit.error(e,"Update DataBean Error:{}",e.getMessage());
//        }
//    }
//
//    /**
//     * 业务对象——查
//     * @param key
//     * @return
//     */
//    public static List<AuthUserEntity> like(String key ){
//        final String key2 = key;
//        try{
//            return accessor.runQueryAction(new DBAction<List<AuthUserEntity>>() {
//                @Override
//                public List<AuthUserEntity> run(DAOContext context) throws Exception {
//                    List<AuthUserEntity> result = new ArrayList<AuthUserEntity>();
//                    List<AuthUserEntity> list = context.getDAO(AuthUserDao.class).likeByKey(key2);
//                    for( AuthUserEntity item : list ){
////                        List<LinkEntity> links = context.getDAO(LinkDao.class).getByKey(item.getDid());
//                        result.add( toDataBean(item,null) );
//                    }
//                    return result;
//                }
//            });
//        }catch(Throwable e){
////            LogKit.error(e,"Get DataBeans{} Error:{}",key,e.getMessage());
//        }
//        return new ArrayList<AuthUserEntity>();
//    }
//
//    /**
//     * 根据pId和type查询
//     * @param pId
//     * @param type
//     * @return
//     */
//    public static List<AuthUserEntity> like(final String pId,final String type){
//        try{
//            return accessor.runQueryAction(new DBAction<List<AuthUserEntity>>() {
//                @Override
//                public List<AuthUserEntity> run(DAOContext context) throws Exception {
//                    List<AuthUserEntity> result = new ArrayList<AuthUserEntity>();
//                    List<AuthUserEntity> list = context.getDAO(AuthUserDao.class).like(pId,type);
//                    for( AuthUserEntity item : list ){
////                        List<LinkEntity> links = context.getDAO(LinkDao.class).getByKey(item.getDid());
//                        result.add( toDataBean(item,null) );
//                    }
//                    return result;
//                }
//            });
//        }catch(Throwable e){
////            LogKit.error(e,"Get DataBeans{} Error:{}",key,e.getMessage());
//        }
//        return new ArrayList<AuthUserEntity>();
//    }
//
//    private static AuthUserEntity loadDemoEntity(AuthUserEntity bean, boolean isAdd ){
//        AuthUserEntity entity = new AuthUserEntity();
////        if(isAdd || StringUtils.isEmpty( bean.getId() ) ){
////            bean.setId(UUIDUtil.generate());
////        }
////        entity.setId( bean.getId()  );
////        entity.setDid( bean.getDid());
////        entity.setCode(bean.getCode());
////        entity.setType(bean.getType());
////        entity.setPid(bean.getPid());
////        entity.setBean( bean.getBean() );
//        return entity;
//    }
//
//    private static AuthUserEntity toDataBean(AuthUserEntity entity, List<AuthUserMacEntity> list ){
//        AuthUserEntity bean = new AuthUserEntity();
////        bean.setId( entity.getId() );
////        bean.setDid( entity.getDid());
////        bean.setCode(entity.getCode());
////        bean.setType(entity.getType());
////        bean.setPid(entity.getPid());
////        bean.setBean( entity.getBean() );
////        bean.setAttr3( toAttrBeans(list) );
//        return bean;
//    }
//
//    private static List<AuthUserEntity> toAttrBeans( List<AuthUserMacEntity> links ){
//        List<AuthUserEntity> list = new ArrayList<AuthUserEntity>();
//        for( AuthUserMacEntity item : links  ){
//            AuthUserEntity bean = new AuthUserEntity();
////            bean.setAttr1(item.getId());
////            bean.setAttr2(item.getAttr3());
//            list.add( bean );
//        }
//        return list;
//    }
//
//    private static List<AuthUserMacEntity> loadLinkEntities(AuthUserEntity bean, boolean isAdd ){
//        List<AuthUserMacEntity> list = new ArrayList<AuthUserMacEntity>();
////        List<DemoBean> attr3 = bean.getAttr3();
////        for( DemoBean item : attr3  ){
////            if(isAdd || StringUtils.isEmpty( item.getAttr1() ) ){
////                item.setAttr1(UUIDUtil.generate());
////            }
////            list.add( new LinkEntity()
////                    .id( item.getAttr1() )
////                    .key( bean.getKey() )
////                    .attr3( item.getAttr2() )
////            );
////        }
//        return list;
//    }

}
