package com.fr.plugin.loginAuth.filter;

import com.fr.base.Base64;
import com.fr.decision.fun.impl.AbstractEmbedRequestFilterProvider;
import com.fr.decision.webservice.utils.WebServiceUtils;
import com.fr.decision.webservice.v10.login.LoginService;
import com.fr.json.JSONObject;
import com.fr.locale.InterProviderFactory;
import com.fr.plugin.loginAuth.dbAccessProvider.AuthUserDBController;
import com.fr.plugin.loginAuth.dbAccessProvider.AuthUserEntity;
import com.fr.plugin.loginAuth.dbAccessProvider.AuthUserMacEntity;
import com.fr.plugin.loginAuth.utils.FRUtils;
import com.fr.plugin.loginAuth.utils.ResponseUtils;
import com.fr.plugin.loginAuth.utils.Utils;
import com.fr.plugin.weixin.server.bean.WeiXinAgent;
import com.fr.plugin.weixin.server.config.WeiXinConfig;
import com.fr.plugin.weixin.server.constant.WeiXinConstants;
import com.fr.plugin.weixin.server.log.WeiXinLogErrorCode;
import com.fr.plugin.weixin.server.log.WeiXinLoggerFactory;
import com.fr.plugin.weixin.server.util.*;
import com.fr.stable.StringUtils;
import com.fr.web.utils.WebUtils;
import org.slf4j.MDC;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.List;
import java.util.UUID;

public class SSOFilter extends AbstractEmbedRequestFilterProvider {

    @Override
    public void filter(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) {
        String url = FRUtils.getAllUrl(httpServletRequest);
        FRUtils.FRLogInfo(url);
        FRUtils.FRLogInfo("pc认证" + url.contains("/decision/login"));

        //PC认证
        if (url.contains("/decision/login")) {
            String username = httpServletRequest.getHeader("username");
            String mac = httpServletRequest.getHeader("mac");

            FRUtils.FRLogInfo("pcParam:username=" + username + ";mac=" + mac);
            if (Utils.isNotNullStr(username)) {
                //没有mac直接加入黑名单
                if (Utils.isNullStr(mac)) {
                    mac = "";

                    AuthUserMacEntity ame = new AuthUserMacEntity();
                    ame.setAuthstatus(1);
                    ame.setDevicemac(mac);
                    ame.setWorkcode(username);
                    ame.setId(UUID.randomUUID().toString());
                    ame.setCreateddate(new Date());
                    ame.setDevicetype("PC");

                    FRUtils.FRLogInfo("添加mac");
                    try {
                        List<AuthUserEntity> authUserMac = AuthUserDBController.getByUserName(username);
                        if (authUserMac != null && authUserMac.size() > 0) {
                            List<AuthUserMacEntity> authUserMacs = AuthUserDBController.getUserAuthMac2(username, mac, "PC");
                            if (authUserMacs != null && authUserMacs.size() <= 0) {
                                AuthUserDBController.add(ame);
                            }
                        }
                    } catch (Exception e) {
                        FRUtils.FRLogInfo("添加mac匹配异常:" + e.getMessage());
                    }

                    JSONObject json = new JSONObject();
                    json.put("errorCode", "90000");
                    json.put("errorMsg", "没有权限登录");

                    ResponseUtils.response(httpServletResponse, json);
                    return;
                }

                //查询用户是否再AuthUser表中，如果在
                List<AuthUserEntity> blackList = null;

                try {
                    blackList = AuthUserDBController.getByUserName(username);
                } catch (Exception e) {
                    FRUtils.FRLogInfo("获取黑名单异常:" + e.getMessage());
                    JSONObject json = new JSONObject();
                    json.put("errorCode", "90000");
                    json.put("errorMsg", "没有权限登录");

                    ResponseUtils.response(httpServletResponse, json);
                    return;
                }

                if (blackList != null && blackList.size() > 0) {
                    FRUtils.FRLogInfo("黑名单:" + blackList.size());
                    FRUtils.FRLogInfo("黑名单:" + blackList.get(0).getWorkcode());
                    //查询AuthUserMac表
                    List<AuthUserMacEntity> authUserMac = null;
                    try {
                        authUserMac = AuthUserDBController.getUserAuthMac2(username, mac, "PC");
                    } catch (Exception e) {
                        FRUtils.FRLogInfo("获取mac匹配异常:" + e.getMessage());
                        JSONObject json = new JSONObject();
                        json.put("errorCode", "90000");
                        json.put("errorMsg", "没有权限登录");

                        ResponseUtils.response(httpServletResponse, json);
                        return;
                    }
                    //如果没有或者只有姓名，则添加一条username
                    if (authUserMac == null || authUserMac.size() <= 0) {
                        AuthUserMacEntity ame = new AuthUserMacEntity();
                        ame.setAuthstatus(1);
                        ame.setDevicemac(mac);
                        ame.setWorkcode(username);
                        ame.setId(UUID.randomUUID().toString());
                        ame.setCreateddate(new Date());
                        ame.setDevicetype("PC");

                        FRUtils.FRLogInfo("添加mac");
                        try {
                            AuthUserDBController.add(ame);
                        } catch (Exception e) {
                            FRUtils.FRLogInfo("添加mac匹配异常:" + e.getMessage());
                        }

                        JSONObject json = new JSONObject();
                        json.put("errorCode", "90000");
                        json.put("errorMsg", "没有权限登录");

                        ResponseUtils.response(httpServletResponse, json);
                        return;
                    }

                    //如果有并且匹配，查询状态并确定是否放行
                    boolean flag = false;

                    for (AuthUserMacEntity ame : authUserMac) {
                        if (ame.getAuthstatus() == 0) {
                            flag = true;
                        }
                    }

                    if (!flag) {
                        JSONObject json = new JSONObject();
                        json.put("errorCode", "90000");
                        json.put("errorMsg", "没有权限登录");

                        ResponseUtils.response(httpServletResponse, json);
                        return;
                    }
                }
            }

        }

        //微信认证
        try {
            boolean isWeixin = url.contains("weixin/single/login");

            //企业微信单点
            if (isWeixin) {
                boolean available = WeiXinUtils.isLicenseAvailable();
                if (available) {
                    MDC.put("logger", WeiXinConstants.LOGIN);
                    FRUtils.FRLogInfo("===开始单点登录===");

                    String decUserName = StringUtils.EMPTY;
                    String code = WebUtils.getHTTPRequestParameter(httpServletRequest, "code");
                    FRUtils.FRLogInfo("code: " + code);

                    String param = WebUtils.getHTTPRequestParameter(httpServletRequest, "sb");
                    FRUtils.FRLogInfo("sb: " + param);

                    String redirectUrl = httpServletRequest.getParameter("redirectUrl"); // 这里不能使用WebUtils的方法，会把中文模板名称的编码给解析了导致找不到模板
                    if (WeiXinConfig.getInstance().getUrlMap().containsKey(redirectUrl)) {
                        redirectUrl = WeiXinConfig.getInstance().getUrlMap().get(redirectUrl);
                    }
                    FRUtils.FRLogInfo("redirectUrl: " + redirectUrl);
                    FRUtils.FRLogInfo("isnotEmpty:" + StringUtils.isNotEmpty(redirectUrl));
                    FRUtils.FRLogInfo("2:" + WeiXinUtils.compareUrlDomain(redirectUrl, WeiXinConfig.getInstance().getUrl()));

                    if (StringUtils.isNotEmpty(redirectUrl) && WeiXinUtils.compareUrlDomain(redirectUrl, WeiXinConfig.getInstance().getUrl())) {
                        FRUtils.FRLogInfo("agent:1");
                        WeiXinAgent agent = WeiXinAgentUtils.getAgentByCheckMD5(param);
                        FRUtils.FRLogInfo("agent:2");
                        String secret = StringUtils.EMPTY;

                        if (agent != null) {
                            secret = agent.getSecret();
                        } else {
                            FRUtils.FRLogInfo("单点登录请求中sb参数不正确，无法找到对应微信应用！" + WeiXinLogErrorCode.LOST_SB_PARAM);
                        }

                        FRUtils.FRLogInfo("secret:" + secret);

                        JSONObject userInfo = WeiXinRequest.getUserInfoByCode(code, secret);
                        FRUtils.FRLogInfo("userInfo:" + userInfo.toString());

                        if (userInfo != null && userInfo.has("UserId")) {
                            String userId = userInfo.optString("UserId");
                            JSONObject detailUserInfo = WeiXinRequest.getDetailUserInfoByUserId(secret, userId);
                            if (WeiXinRequest.isOk(detailUserInfo)) {
                                FRUtils.FRLogInfo(String.format("userid为%s的用户详细信息为：", userId) + detailUserInfo.toString());
                                if (WeiXinUserUtils.isWeiXinUserEnable(detailUserInfo)) {
                                    WeiXinLoggerFactory.getLogger().debug("开始匹配平台用户。。。");
                                    decUserName = WeiXinUserUtils.weiXinUserToDecUser(detailUserInfo);
                                } else {
                                    FRUtils.FRLogInfo("微信用户被禁用，请于微信管理后台设置！" + WeiXinLogErrorCode.WEIXIN_USER_DISABLE);
                                }
                            } else {
                                FRUtils.FRLogInfo(String.format("获取userId为%s的用户详细信息失败！", userId) + WeiXinLogErrorCode.GET_USER_DETAIL_FAIL);
                            }
                        } else {
                            FRUtils.FRLogInfo("code换取userId失败，请检查网络设置！" + WeiXinLogErrorCode.GET_USERID_FAIL);
                        }

                        if (WeiXinDecUserUtils.isUserExist(decUserName)) {
                            FRUtils.FRLogInfo("decUserName:"+decUserName);
                            boolean isLogin = allowLogin(decUserName,userInfo);
                            if(!isLogin){
                                ResponseUtils.failedResponse(httpServletResponse,"您的电脑未授权，请联系管理员");
                                return ;
                            }
                            String token = LoginService.getInstance().login(httpServletRequest, httpServletResponse, decUserName, StringUtils.EMPTY, WeiXinConstants.WEIXIN_PLUGIN_ID);
                            FRUtils.FRLogInfo("单点登录成功！生成token为：" + token);
                        } else if (StringUtils.isNotEmpty(decUserName)) {
                            FRUtils.FRLogInfo(String.format("平台用户中无%s用户！", decUserName) + WeiXinLogErrorCode.NO_DEC_USER);
                        }

                        String jsApiPath = Base64.encode(WeiXinConstants.WEIXIN_JSAPI_PATH.getBytes("UTF-8"));
                        redirectUrl = WeiXinURLUtils.putParamToUrl(redirectUrl, "js_api_path", jsApiPath);

                        redirectUrl = redirectUrl + "&sb=" + param;

                        FRUtils.FRLogInfo("开始跳转url：" + redirectUrl);
                        httpServletResponse.sendRedirect(redirectUrl);
                        FRUtils.FRLogInfo("===单点登录结束===");
                        MDC.clear();
                    } else {
                        FRUtils.FRLogInfo("进入了else2");
                        WebUtils.printAsString(httpServletResponse, WebServiceUtils.generateUnavailableWebPage(
                                InterProviderFactory.getProvider().getLocText("Dec-WeiXin_Not_Trusted_Domain_Error"),
                                "",
                                InterProviderFactory.getProvider().getLocText("Dec-WeiXin_Not_Trusted_Domain_Exception")
                        ));
                    }
                } else {
                    FRUtils.FRLogInfo("进入了else");
                    WeiXinUtils.routeToExpirePage(httpServletRequest, httpServletResponse);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    private static boolean allowLogin(String userName,JSONObject userInfo) {
        //查询用户是否再AuthUser表中，如果在
        List<AuthUserEntity> blackList = null;

        try {
            blackList = AuthUserDBController.getByUserName(userName);
        } catch (Exception e) {
            FRUtils.FRLogInfo("获取黑名单异常:" + e.getMessage());
            return false;
        }

        if (blackList != null && blackList.size() > 0) {
            FRUtils.FRLogInfo("blackListSize:" + blackList.size());
            String mac = userInfo.getString("DeviceId");

            //查询AuthUserMac表
            List<AuthUserMacEntity> authUserMac = null;
            try {
                authUserMac = AuthUserDBController.getUserAuthMac2(userName, mac, "mobile");
            } catch (Exception e) {
                FRUtils.FRLogInfo("获取mac匹配异常:" + e.getMessage());
                return false;
            }

            //如果没有或者只有姓名，则添加一条
            if (authUserMac == null || authUserMac.size() <= 0) {
                AuthUserMacEntity ame = new AuthUserMacEntity();
                ame.setAuthstatus(1);
                ame.setDevicemac(mac);
                ame.setWorkcode(userName);
                ame.setId(UUID.randomUUID().toString());
                ame.setCreateddate(new Date());
                ame.setDevicetype("mobile");
                try {
                    AuthUserDBController.add(ame);
                } catch (Exception e) {
                    FRUtils.FRLogInfo("添加mac匹配异常:" + e.getMessage());
                }

                return false;
            }

            FRUtils.FRLogInfo("authUserMac:" + authUserMac.size());

            //如果有并且匹配，查询状态并确定是否放行
            boolean flag = false;

            for (AuthUserMacEntity ame : authUserMac) {
                if (ame.getAuthstatus() == 0) {
                    flag = true;
                }
            }

            return flag;
        }

        return true;
    }

}