package com.fr.plugin.loginAuth.httpAuth;

import com.fr.decision.authority.data.User;
import com.fr.decision.fun.impl.AbstractHttpAuthorizeProvider;
import com.fr.json.JSONObject;
import com.fr.plugin.loginAuth.dbAccessProvider.AuthUserDBController;
import com.fr.plugin.loginAuth.dbAccessProvider.AuthUserEntity;
import com.fr.plugin.loginAuth.dbAccessProvider.AuthUserMacEntity;
import com.fr.plugin.loginAuth.utils.CipherUtils;
import com.fr.plugin.loginAuth.utils.FRUtils;
import com.fr.plugin.loginAuth.utils.HttpUtils;
import com.fr.plugin.loginAuth.utils.Utils;
import com.fr.plugin.transform.FunctionRecorder;
import java.util.List;
import java.util.UUID;

@FunctionRecorder
public class Login extends AbstractHttpAuthorizeProvider {

    @Override
    public Scope scope() {
        return Scope.REPLACE;
    }

    @Override
    public boolean authorize(String userName, String psd, String s2, String s3) {
        //查询用户是否再AuthUser表中，如果在
        List<AuthUserEntity> blackList = null;

        try {
            blackList = AuthUserDBController.getByUserName(userName);
        } catch (Exception e) {
            FRUtils.FRLogInfo("获取黑名单异常:"+e.getMessage());
            return false;
        }

        if(blackList != null && blackList.size() > 0){
            String mac = getMac();
            if(Utils.isNullStr(mac)){
                return false;
            }
            //查询AuthUserMac表
            List<AuthUserMacEntity> authUserMac = null;
            try {
                authUserMac = AuthUserDBController.getUserAuthMac(userName,mac);
            } catch (Exception e) {
                FRUtils.FRLogInfo("获取mac匹配异常:"+e.getMessage());
                return false;
            }
            //如果没有或者只有姓名，则添加一条
            if(authUserMac == null || authUserMac.size() <= 0){
                AuthUserMacEntity ame = new AuthUserMacEntity();
                ame.setAuthstatus(1);
                ame.setDevicemac(mac);
                ame.setWorkcode(userName);
                ame.setId(UUID.randomUUID().toString());
                try {
                    AuthUserDBController.add(ame);
                } catch (Exception e) {
                    FRUtils.FRLogInfo("添加mac匹配异常:"+e.getMessage());
                }
                return false;
            }

            //如果有并且匹配，查询状态并确定是否放行
            boolean flag = false;

            for(AuthUserMacEntity ame : authUserMac){
                if(ame.getAuthstatus() == 0){
                    flag = true;
                }
            }

            if(!flag){
                return false;
            }
        }

        String password = CipherUtils.jdksha256(psd);

        User user = FRUtils.getFRUserByUserName(userName);
        return password.equals(user.getPassword());

//

    }
    @Override
    public boolean authorize(String s, String s1) {
        return false;
    }


    private String getMac(){
        String url = "http://localhost:9090/domain/getDomain";

        String result = HttpUtils.get(url,null,null);

        if(Utils.isNullStr(result)){
            return "";
        }

        JSONObject json = new JSONObject(result);
        boolean success = json.getBoolean("success");

        if(!success){
            return "";
        }

        return json.getString("domain");
    }
}
